<?php

namespace Drupal\kinescope\Plugin\media\Source;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\FieldTypePluginManagerInterface;
use Drupal\Core\File\FileExists;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\StreamWrapper\StreamWrapperManagerInterface;
use Drupal\media\MediaInterface;
use Drupal\media\MediaSourceBase;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\RequestException;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Kinescope Media Source.
 *
 * @MediaSource(
 *   id = "kinescope_media",
 *   label = @Translation("Kinescope"),
 *   description = @Translation("Kinescope media."),
 *   allowed_field_types = {"kinescope"},
 *   default_thumbnail_filename = "kinescope.png"
 * )
 */
class Kinescope extends MediaSourceBase {

  const META_THUMBNAIL = 'thumbnail_uri';
  const META_VIDEO_URL = 'video_url';

  /**
   * The file system.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\Client
   */
  protected $httpClient;

  /**
   * Stream wrapper manager.
   *
   * @var \Drupal\Core\StreamWrapper\StreamWrapperManagerInterface
   */
  protected $streamWrapperManager;

  /**
   * Logger channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * Construct.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    EntityTypeManagerInterface $entity_type_manager,
    EntityFieldManagerInterface $entity_field_manager,
    FieldTypePluginManagerInterface $field_type_manager,
    ConfigFactoryInterface $config_factory,
    StreamWrapperManagerInterface $stream_wrapper_manager,
    FileSystemInterface $file_system,
    ClientInterface $http_client,
    LoggerChannelFactoryInterface $logger_channel_factory,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $entity_type_manager, $entity_field_manager, $field_type_manager, $config_factory);
    $this->streamWrapperManager = $stream_wrapper_manager;
    $this->fileSystem = $file_system;
    $this->httpClient = $http_client;
    $this->logger = $logger_channel_factory->get('kinescope');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('entity_field.manager'),
      $container->get('plugin.manager.field.field_type'),
      $container->get('config.factory'),
      $container->get('stream_wrapper_manager'),
      $container->get('file_system'),
      $container->get('http_client'),
      $container->get('logger.factory'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getMetadataAttributes() {
    return [
      self::META_THUMBNAIL => $this->t('Thumbnail URI'),
      self::META_VIDEO_URL => $this->t('Video URL'),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getMetadata(MediaInterface $media, $attribute_name) {

    if ($attribute_name != self::META_THUMBNAIL) {
      return parent::getMetadata($media, $attribute_name);
    }

    // Get the file and image data.
    /** @var \Drupal\Core\Field\FieldItemListInterface $sourceField */
    $sourceField = $media->get($this->configuration['source_field']);

    // If the source field is not required, it may be empty.
    if (!$sourceField || $sourceField->isEmpty()) {
      return parent::getMetadata($media, $attribute_name);
    }

    $video_url = $sourceField->getValue()[0]['video_url'];
    $video_id = $sourceField->getValue()[0]['video_id'];
    $host = 'https://kinescope.io';

    try {
      $directory = 'public://kinescope_thumbs/';
      $thumb_uri = "$directory/$video_id.jpg";

      try {
        $response = $this->httpClient->get("$host/$video_id/poster.webp");
        if ($response->getStatusCode() !== 200) {
          $this->logger->warning('Could not get remote thumbnail from {video_url}.', [
            'video_url' => $video_url,
          ]);
          return parent::getMetadata($media, $attribute_name);
        }

        $this->fileSystem->saveData((string) $response->getBody(), $thumb_uri, FileExists::Rename);
        return $thumb_uri;
      }
      catch (RequestException $e) {
        $this->logger->warning('Could not download remote thumbnail from {video_url}.', [
          'video_url' => $video_url,
        ]);
      }

      return parent::getMetadata($media, $attribute_name);
    }
    catch (\Exception $e) {
      $this->logger->error($e->getMessage() . ' for the video url: {video_url}', [
        'video_url' => $video_url,
      ]);
      throw new \Exception('Video not found');
    }

    return NULL;
  }

}
